/* Copyright (C) 2014 by Bundesamt für Sicherheit in der Informationstechnik
 * Software engineering by Intevation GmbH
 *
 * This file is Free Software under the GNU GPL (v>=2)
 * and comes with ABSOLUTELY NO WARRANTY!
 * See LICENSE.txt for details.
 */
#include "mainwindow.h"
#include "processhelp.h"
#include "logging.h"
#include "selftest.h"
#include "util.h"
#ifdef WIN32
#include "taskscheduler.h"
#endif

#include <QApplication>
#include <QFile>
#include <QtPlugin>
#include <QMessageBox>
#include <QSettings>
#include <QDebug>
#include <QTranslator>

#include <QStyleFactory>
#include <QFontDatabase>

#ifndef VERSION
#define VERSION "0.0.1"
#endif

#ifndef APPNAME
#define APPNAME "TrustBridge"
#endif

#ifndef STARTUP_FILE_NAME
#define STARTUP_FILE_NAME "trustbridge-tray-starter.cfg"
#endif

#ifndef ORGANIZATION
#define ORGANIZATION "BSI"
#endif

#define COPYRIGHT "Copyright (C) 2014 by Bundesamt für Sicherheit in der Informationstechnik \n" \
                  "Software engineering by Intevation GmbH \n\n" \
                  "This file is Free Software under the GNU GPL (v>=2)\n" \
                  "and comes with ABSOLUTELY NO WARRANTY!\n"

#ifdef Q_OS_WIN
 Q_IMPORT_PLUGIN(QWindowsIntegrationPlugin)
#else
 Q_IMPORT_PLUGIN(QXcbIntegrationPlugin)
#endif

#ifdef IS_TAG_BUILD
bool g_debug = false;
#else
bool g_debug = true;
#endif

QtMessageHandler g_default_msg_handler = NULL;

void filterDebugOutput(QtMsgType type, const QMessageLogContext &context, const QString &msg)
{
    if (!g_debug && type == QtDebugMsg) {
        return;
    }

    if (g_default_msg_handler) {
        (*g_default_msg_handler)(type, context, msg);
    }
}


int main(int argc, char **argv)
{
  time_t signed_time = 0;
#ifdef WIN32
    /* First verify integrity even before calling QApplication.
     * We only do this on Windows as we have a PKCS#7 embedded
     * signature there which we check with OS methods.
     *
     * On GNU/Linux platforms you should use an IDS system to
     * monitor executable corruptions.
     */
    if (!selftest(&signed_time)) {
        syslog_error_printf("Integrity check failed.");
        MessageBoxW(NULL,
                L"TrustBridge wurde nach der Installation modifizert.\n"
                L"Um ihr System zu schützen wurde das Starten der Anwendung abgebrochen.\n"
                L"Bitte installieren Sie TrustBridge erneut.\n\n"
                L"TrustBridge has been modified after installation.\n"
                L"To protect your system the application launch has been aborted.\n"
                L"Please reinstall TrustBridge.",
                L"Integritätsprüfung fehlgeschlagen / Integrity check failed.",
                MB_ICONERROR | MB_OK);
        return -1;
    }
#endif

    QApplication app (argc, argv);

    QApplication::setQuitOnLastWindowClosed(false);
    QApplication::setOrganizationName(QString::fromLatin1(ORGANIZATION));
    QApplication::setApplicationName(QString::fromLatin1(APPNAME));
    QApplication::setApplicationVersion(QString::fromLatin1(VERSION));
    QSettings::setDefaultFormat(QSettings::IniFormat);
    QStringList arguments = QApplication::arguments();

    if (arguments.contains("--debug")) {
        g_debug = true;
    }
    g_default_msg_handler = qInstallMessageHandler(filterDebugOutput);

    qDebug() << "Application style is: " << app.style()->metaObject()->className();
    qDebug() << "Available styles: " << QStyleFactory::keys().join(", ");
    qDebug() << "Font is: " << app.font();

    bool trayMode = arguments.contains("--tray");

    if (arguments.contains("--version")) {
        printf (APPNAME " Version: %s \n",
                QApplication::applicationVersion().toLocal8Bit().constData());
#ifndef RELEASE_BUILD
        printf ("Test Version! - Not for productive use!\n";
#endif
        printf (COPYRIGHT);
        return 0;
    }

    QSettings settings;
    settings.beginGroup("settings");
    bool autoStart = settings.value("autostart", true).toBool();
    settings.endGroup();

    if (trayMode && !autoStart) {
        return 0;
    }

    if (ProcessHelp::otherProcessesExist(APPNAME)) {
        qDebug() << "Another " << APPNAME << " process is already running. Exiting.";
        ProcessHelp::activateWindowForProcess(APPNAME);
        return 0;
    }

    QTranslator translator;
    if (QLocale::system().name() == "C") {
        /* Useful for testing / development as the primary target is german */
        translator.load(":/l10n/trustbridge_de_DE");
    } else {
        qDebug() << "Loading translations for: " << "trustbridge_" +
            QLocale::system().name();
        if (!translator.load(":/l10n/trustbridge_" + QLocale::system().name())) {
            qDebug() << "Failed.";
        }
    }
    app.installTranslator(&translator);
/*
    if ((!QSystemTrayIcon::isSystemTrayAvailable() ||
            !QSystemTrayIcon::supportsMessages()) && trayMode) {
        QMessageBox::critical(0, QString::fromLatin1(APPNAME),
                              QObject::tr("Couldn't detect any system tray "
                                          "on this system. This software can only "
                                          "be used in a desktop environment."));
        return 1;
    }
*/
#ifdef Q_OS_WIN
    {
        TaskScheduler taskSched;
        qDebug() << " task sched done: " << taskSched.createDailyTask(QCoreApplication::applicationFilePath(),
                QString::fromLatin1("--tray"), QTime::currentTime());
    }
#else
    /* Save the display on start so that it might later be used by the cron startup script. */
    QFile startup_file (settings.fileName().replace(APPNAME ".ini", STARTUP_FILE_NAME));
    startup_file.open(QIODevice::WriteOnly);
    startup_file.write(QString::fromLatin1("DISPLAY=%1\n").arg(qgetenv("DISPLAY").constData()).toUtf8());
    startup_file.close();
#endif

    /* Install static fonts */

    /* The putenv here works around a bug in qt (#29192). Qt thinks it is a fatal
     * error if the font directory does not exist. */
    qputenv("QT_QPA_FONTDIR", get_install_dir());
    int fontId = QFontDatabase::addApplicationFont(":/fonts/DejaVuSans.ttf");
    if (fontId != -1)
    {
        QFont font("DejaVuSans");
        font.setPointSize(9);
        app.setFont(font);
    }


    QDateTime sigDt;
#ifndef Q_OS_WIN
    /* Read it from the install config. */
    sigDt = ProcessHelp::getSigDtFromInstSettings();
    Q_UNUSED(signed_time);
#else
    if (signed_time != 0 && signed_time != -1) {
        /* Take it from the selftest result. */
        sigDt = QDateTime::fromTime_t(signed_time);
    }
#endif

    MainWindow mainWin(trayMode, sigDt);

    return app.exec();
}
